@@ -1,29 +1,8 @@
-# Copyright (c) 2022 m-col
-# Copyright (c) 2023 elParaguayo
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
 import cairocffi
-from wlroots.util.region import PixmanRegion32
 
 from libqtile.backend.base import drawer
 
@@ -65,17 +44,21 @@ class Drawer(drawer.Drawer):
         if height > self._win.height - offsety:
             height = self._win.height - offsety
 
+        scale = self._win.scale
+
         # Paint recorded operations to our window's underlying ImageSurface
-        with cairocffi.Context(self._win.surface) as context:
+        # Allocation could have failed or surface may have been destroyed, NULL check
+        if not self._win.surface:
+            return
+        surface = cairocffi.Surface._from_pointer(self._win.surface, True)  # type: ignore[attr-defined]
+        with cairocffi.Context(surface) as context:
             context.set_operator(cairocffi.OPERATOR_SOURCE)
+            # Scale the cairo surface by its output (display) scale
+            context.scale(scale, scale)
             # Adjust the source surface position by src_x and src_y e.g. if we want
             # to render part of the surface in a different position
             context.set_source_surface(self.surface, offsetx - src_x, offsety - src_y)
             context.rectangle(offsetx, offsety, width, height)
             context.fill()
 
-        damage = PixmanRegion32()
-        damage.init_rect(offsetx, offsety, width, height)
-        # TODO: do we really need to `set_buffer` here? would be good to just set damage
-        self._win._scene_buffer.set_buffer_with_damage(self._win.wlr_buffer, damage)
-        damage.fini()
+        self._win.set_buffer_with_damage(offsetx, offsety, width, height)
