@@ -5,7 +5,6 @@ import contextlib
 import inspect
 import traceback
 from itertools import islice
-from types import FunctionType
 from typing import TYPE_CHECKING
 
 import xcffib
@@ -34,26 +33,6 @@ HeightValue = 0x0008
 AllValues = 0x000F
 XNegative = 0x0010
 YNegative = 0x0020
-InputHint = 1 << 0
-StateHint = 1 << 1
-IconPixmapHint = 1 << 2
-IconWindowHint = 1 << 3
-IconPositionHint = 1 << 4
-IconMaskHint = 1 << 5
-WindowGroupHint = 1 << 6
-MessageHint = 1 << 7
-UrgencyHint = 1 << 8
-AllHints = (
-    InputHint
-    | StateHint
-    | IconPixmapHint
-    | IconWindowHint
-    | IconPositionHint
-    | IconMaskHint
-    | WindowGroupHint
-    | MessageHint
-    | UrgencyHint
-)
 
 WithdrawnState = 0
 DontCareState = 0
@@ -91,8 +70,10 @@ _NET_WM_STATE_TOGGLE = 2
 
 
 def _geometry_getter(attr):
+    attr_name = "_" + attr
+
     def get_attr(self):
-        if getattr(self, "_" + attr) is None:
+        if getattr(self, attr_name) is None:
             g = self.window.get_geometry()
             # trigger the geometry setter on all these
             self.x = g.x
@@ -100,12 +81,14 @@ def _geometry_getter(attr):
             self.width = g.width
             self.height = g.height
             self.depth = g.depth
-        return getattr(self, "_" + attr)
+        return getattr(self, attr_name)
 
     return get_attr
 
 
 def _geometry_setter(attr):
+    attr_name = "_" + attr
+
     def f(self, value):
         if not isinstance(value, int):
             frame = inspect.currentframe()
@@ -113,7 +96,7 @@ def _geometry_setter(attr):
             logger.error("!!!! setting %s to a non-int %s; please report this!", attr, value)
             logger.error("".join(stack_trace[:-1]))
             value = int(value)
-        setattr(self, "_" + attr, value)
+        setattr(self, attr_name, value)
 
     return f
 
@@ -189,7 +172,7 @@ class XWindow:
             return {
                 "flags": flags,
                 "input": atoms_list[1] if "InputHint" in flags else None,
-                "initial_state": atoms_list[2] if "StateHing" in flags else None,
+                "initial_state": atoms_list[2] if "StateHint" in flags else None,
                 "icon_pixmap": atoms_list[3] if "IconPixmapHint" in flags else None,
                 "icon_window": atoms_list[4] if "IconWindowHint" in flags else None,
                 "icon_x": atoms_list[5] if "IconPositionHint" in flags else None,
@@ -820,6 +803,10 @@ class _Window:
     def get_pid(self):
         return self.window.get_net_wm_pid()
 
+    @property
+    def fullscreen(self):
+        return self._float_state == FloatStates.FULLSCREEN
+
     @expose_command()
     def place(
         self,
@@ -1255,7 +1242,7 @@ class _Window:
     def can_steal_focus(self, can_steal_focus: bool) -> None:
         self._can_steal_focus = can_steal_focus
 
-    def _do_focus(self):
+    def _do_focus(self) -> bool:
         """
         Focus the window if we can, and return whether or not it was successful.
         """
@@ -1540,6 +1527,7 @@ class Internal(_Window, base.Internal):
         if self.window.wid in self.qtile.windows_map:
             # It will be present during config reloads; absent during shutdown as this
             # will follow graceful_shutdown
+            del self.qtile.windows_map[self.wid]
             with contextlib.suppress(xcffib.ConnectionException):
                 self.qtile.core.conn.conn.core.DestroyWindow(self.window.wid)
 
@@ -2153,59 +2141,11 @@ class Window(_Window, base.Window):
             self.window.set_property("_NET_WM_STATE", list(current_state))
         elif atoms["_NET_ACTIVE_WINDOW"] == opcode:
             source = data.data32[0]
-            if source == 2:  # XCB_EWMH_CLIENT_SOURCE_TYPE_NORMAL
+            if source == 2:  # Request from a pager should immediately focus the window
                 logger.debug("Focusing window by pager")
-                self.qtile.current_screen.set_group(self.group)
-                self.group.focus(self)
-                self.bring_to_front()
-            else:  # XCB_EWMH_CLIENT_SOURCE_TYPE_OTHER
-                focus_behavior = self.qtile.config.focus_on_window_activation
-                if (
-                    focus_behavior == "focus"
-                    or type(focus_behavior) is FunctionType
-                    and focus_behavior(self)
-                ):
-                    logger.debug("Focusing window")
-                    # Windows belonging to a scratchpad need to be toggled properly
-                    if isinstance(self.group, ScratchPad):
-                        for dropdown in self.group.dropdowns.values():
-                            if dropdown.window is self:
-                                dropdown.show()
-                                break
-                    else:
-                        self.qtile.current_screen.set_group(self.group)
-                        self.group.focus(self)
-                elif focus_behavior == "smart":
-                    if not self.group.screen:
-                        logger.debug(
-                            "Ignoring focus request (focus_on_window_activation='smart')"
-                        )
-                        return
-                    if self.group.screen == self.qtile.current_screen:
-                        logger.debug("Focusing window")
-                        # Windows belonging to a scratchpad need to be toggled properly
-                        if isinstance(self.group, ScratchPad):
-                            for dropdown in self.group.dropdowns.values():
-                                if dropdown.window is self:
-                                    dropdown.show()
-                                    break
-                        else:
-                            self.qtile.current_screen.set_group(self.group)
-                            self.group.focus(self)
-                    else:  # self.group.screen != self.qtile.current_screen:
-                        logger.debug("Setting urgent flag for window")
-                        self.urgent = True
-                        hook.fire("client_urgent_hint_changed", self)
-                elif focus_behavior == "urgent":
-                    logger.debug("Setting urgent flag for window")
-                    self.urgent = True
-                    hook.fire("client_urgent_hint_changed", self)
-                elif focus_behavior == "never":
-                    logger.debug("Ignoring focus request (focus_on_window_activation='never')")
-                else:
-                    logger.debug(
-                        "Invalid value for focus_on_window_activation: %s", focus_behavior
-                    )
+                self.activate()
+            else:  # Request from the application
+                self.activate_by_config()
         elif atoms["_NET_CLOSE_WINDOW"] == opcode:
             self.kill()
         elif atoms["WM_CHANGE_STATE"] == opcode:
