@@ -2,7 +2,6 @@ import atexit
 import enum
 import os
 import sys
-from typing import Optional
 
 from libqtile.log_utils import logger
 
@@ -20,12 +19,13 @@ class LifeCycle:
     # referenced here when atexit fires will NOT be finalized properly.
     def __init__(self) -> None:
         self.behavior = Behavior.NONE
-        self.state_file: Optional[str] = None
+        self.exitcode: int = 0
+        self.state_file: str | None = None
         atexit.register(self._atexit)
 
     def _atexit(self) -> None:
         if self.behavior is Behavior.RESTART:
-            argv = [sys.executable] + sys.argv
+            argv = sys.argv
             if "--no-spawn" not in argv:
                 argv.append("--no-spawn")
             argv = [s for s in argv if not s.startswith("--with-state")]
@@ -33,9 +33,13 @@ class LifeCycle:
                 argv.append("--with-state=" + self.state_file)
             logger.warning("Restarting Qtile with os.execv(...)")
             # No other code will execute after the following line does
-            os.execv(sys.executable, argv)
+            os.execv(argv[0], argv)
         elif self.behavior is Behavior.TERMINATE:
             logger.warning("Qtile will now terminate")
+            # the if statement should be unnecessary, but keeps code coverage working
+            # calling os._exit prevents later injected atexit handlers from running
+            if self.exitcode:
+                os._exit(self.exitcode)
         elif self.behavior is Behavior.NONE:
             pass
 
