@@ -1,23 +1,3 @@
-# Copyright (c) 2008, Aldo Cortesi. All rights reserved.
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 from __future__ import annotations
 
 import asyncio
@@ -29,9 +9,10 @@ import pickle
 import shlex
 import shutil
 import signal
+import socket
 import subprocess
-import sys
 import tempfile
+import time
 from collections import defaultdict
 from logging.handlers import RotatingFileHandler
 from pathlib import Path
@@ -47,16 +28,23 @@ from libqtile.command.interface import IPCCommandServer, QtileCommandInterface
 from libqtile.config import Click, Drag, Key, KeyChord, Match, Mouse, Rule, Screen, ScreenRect
 from libqtile.config import ScratchPad as ScratchPadConfig
 from libqtile.core.lifecycle import lifecycle
-from libqtile.core.loop import LoopContext, QtileEventLoopPolicy
+from libqtile.core.loop import LoopContext
 from libqtile.core.state import QtileState
 from libqtile.dgroups import DGroups
 from libqtile.extension.base import _Extension
 from libqtile.group import _Group
+from libqtile.interactive.repl import repl_server
 from libqtile.log_utils import logger
 from libqtile.resources.sleep import inhibitor
 from libqtile.scratchpad import ScratchPad
 from libqtile.scripts.main import VERSION
-from libqtile.utils import cancel_tasks, get_cache_dir, lget, remove_dbus_rules, send_notification
+from libqtile.utils import (
+    create_task,
+    get_cache_dir,
+    lget,
+    remove_dbus_rules,
+    send_notification,
+)
 from libqtile.widget.base import _Widget
 
 if TYPE_CHECKING:
@@ -105,11 +93,22 @@ class Qtile(CommandObject):
         self.screens: list[Screen] = []
 
         libqtile.init(self)
+        libqtile.event_loop = asyncio.new_event_loop()
 
-        self._stopped_event: asyncio.Event | None = None
+        self._stopped_event: asyncio.Event = asyncio.Event()
 
         self.server = IPCCommandServer(self)
 
+        self.locked = False
+        hook.subscribe.locked(self.lock)
+        hook.subscribe.unlocked(self.unlock)
+
+    def lock(self) -> None:
+        self.locked = True
+
+    def unlock(self) -> None:
+        self.locked = False
+
     def load_config(self, initial: bool = False) -> None:
         try:
             self.config.load()
@@ -148,13 +147,6 @@ class Qtile(CommandObject):
         for button in self.config.mouse:
             self.grab_button(button)
 
-        # no_spawn is set after the very first startup; we only want to run the
-        # startup hook once.
-        if not self.no_spawn:
-            hook.fire("startup_once")
-            self.no_spawn = True
-        hook.fire("startup")
-
         if self._state:
             if isinstance(self._state, str):
                 try:
@@ -170,6 +162,18 @@ class Qtile(CommandObject):
 
         self.core.on_config_load(initial)
 
+        if self.config.reconfigure_screens:
+            hook.subscribe.screen_change(self.reconfigure_screens)
+
+        # no_spawn is set after the very first startup; we only want to run the
+        # startup hook once. Note that this needs to happen *after* the config
+        # is loaded and we have subscribed reconfigure_screens() in case people
+        # do xrandr style manipulation in this hook.
+        if not self.no_spawn:
+            hook.fire("startup_once")
+            self.no_spawn = True
+        hook.fire("startup")
+
         if self._state:
             for screen in self.screens:
                 screen.group.layout.show(screen.get_rect())
@@ -178,9 +182,6 @@ class Qtile(CommandObject):
         self.update_desktops()
         hook.subscribe.setgroup(self.update_desktops)
 
-        if self.config.reconfigure_screens:
-            hook.subscribe.screen_change(self.reconfigure_screens)
-
         # Start the sleep inhibitor process to listen to sleep signals
         # NB: the inhibitor will only connect to the dbus service if the
         # user has used the "suspend" or "resume" hooks in their config.
@@ -202,7 +203,7 @@ class Qtile(CommandObject):
         return socket_path
 
     def loop(self) -> None:
-        asyncio.run(self.async_loop())
+        asyncio.run(self.async_loop(), loop_factory=lambda: libqtile.event_loop)
 
     async def async_loop(self) -> None:
         """Run the event loop
@@ -210,9 +211,6 @@ class Qtile(CommandObject):
         Finalizes the Qtile instance on exit.
         """
         self._eventloop = asyncio.get_running_loop()
-        # Set the event loop policy to facilitate access to main event loop
-        asyncio.set_event_loop_policy(QtileEventLoopPolicy(self))
-        self._stopped_event = asyncio.Event()
         self.core.qtile = self
         self.load_config(initial=True)
         self.core.setup_listener()
@@ -221,6 +219,7 @@ class Qtile(CommandObject):
         faulthandler.register(signal.SIGUSR2, all_threads=True)
 
         try:
+            signals: dict[signal.Signals, Callable]
             signals = {
                 signal.SIGTERM: self.stop,
                 signal.SIGINT: self.stop,
@@ -240,6 +239,8 @@ class Qtile(CommandObject):
                 ),
             ):
                 await self._stopped_event.wait()
+                if lifecycle.behavior != lifecycle.behavior.RESTART:
+                    await self.graceful_shutdown()
         finally:
             self.finalize()
             self.core.remove_listener()
@@ -248,9 +249,27 @@ class Qtile(CommandObject):
         hook.fire("shutdown")
         lifecycle.behavior = lifecycle.behavior.TERMINATE
         lifecycle.exitcode = exitcode
-        self.core.graceful_shutdown()
         self._stop()
 
+    def normal_windows(self) -> list[base.WindowType]:
+        return list(
+            filter(lambda w: isinstance(w, base.Window), self.windows_map.copy().values())
+        )
+
+    async def graceful_shutdown(self) -> None:
+        """Try to close windows gracefully before exiting"""
+        # Copy in case the dictionary changes during the loop
+        for win in self.normal_windows():
+            win.kill()
+
+        # give everyone a little time to exit and write their state. but don't
+        # sleep forever (1s).
+        end = time.time() + 1
+        while time.time() < end:
+            await asyncio.sleep(0.1)
+            if len(self.normal_windows()) == 0:
+                break
+
     @expose_command()
     def restart(self) -> None:
         """Restart Qtile.
@@ -276,8 +295,7 @@ class Qtile(CommandObject):
 
     def _stop(self) -> None:
         logger.debug("Stopping qtile")
-        if self._stopped_event is not None:
-            self._stopped_event.set()
+        self._stopped_event.set()
 
     def dump_state(self, buf: Any) -> None:
         try:
@@ -331,6 +349,7 @@ class Qtile(CommandObject):
 
             for screen in self.screens:
                 screen.finalize_gaps()
+
         except:  # noqa: E722
             logger.exception("exception during finalize")
         hook.clear()
@@ -339,7 +358,6 @@ class Qtile(CommandObject):
         self._finalize_configurables()
         remove_dbus_rules()
         inhibitor.stop()
-        cancel_tasks()
         self.core.finalize()
 
     def add_autogen_group(self, screen_idx: int) -> _Group:
@@ -430,9 +448,7 @@ class Qtile(CommandObject):
 
         for screen in self.screens:
             if screen not in screens:
-                for gap in screen.gaps:
-                    if isinstance(gap, bar.Bar) and gap.window:
-                        gap.finalize()
+                screen.finalize_gaps()
 
         self.screens = screens
 
@@ -735,28 +751,24 @@ class Qtile(CommandObject):
         if self.current_screen and isinstance(win, base.Window):
             # Window may have been bound to a group in the hook.
             if not win.group and self.current_screen.group:
-                self.current_screen.group.add(win, focus=win.can_steal_focus)
+                self.current_screen.group.add(win)
 
         hook.fire("client_managed", win)
 
     def unmanage(self, wid: int) -> None:
         c = self.windows_map.get(wid)
         if c:
-            group = None
+            # Fire the hook before removing the group from the window so hooked
+            # functions can access it
+            hook.fire("client_killed", c)
             if isinstance(c, base.Static):
                 if c.reserved_space:
                     self.free_reserved_space(c.reserved_space, c.screen)
             elif isinstance(c, base.Window):
                 if c.group:
-                    group = c.group
                     c.group.remove(c)
             del self.windows_map[wid]
 
-            if isinstance(c, base.Window):
-                # Put the group back on the window so hooked functions can access it.
-                c.group = group
-            hook.fire("client_killed", c)
-
     def find_screen(self, x: int, y: int) -> Screen | None:
         """Find a screen based on the x and y offset"""
         result = []
@@ -1020,30 +1032,35 @@ class Qtile(CommandObject):
     def debug(self) -> None:
         """Set log level to DEBUG"""
         logger.setLevel(logging.DEBUG)
+        self.core.update_backend_log_level()
         logger.debug("Switching to DEBUG threshold")
 
     @expose_command()
     def info(self) -> None:
         """Set log level to INFO"""
         logger.setLevel(logging.INFO)
+        self.core.update_backend_log_level()
         logger.info("Switching to INFO threshold")
 
     @expose_command()
     def warning(self) -> None:
         """Set log level to WARNING"""
         logger.setLevel(logging.WARNING)
+        self.core.update_backend_log_level()
         logger.warning("Switching to WARNING threshold")
 
     @expose_command()
     def error(self) -> None:
         """Set log level to ERROR"""
         logger.setLevel(logging.ERROR)
+        self.core.update_backend_log_level()
         logger.error("Switching to ERROR threshold")
 
     @expose_command()
     def critical(self) -> None:
         """Set log level to CRITICAL"""
         logger.setLevel(logging.CRITICAL)
+        self.core.update_backend_log_level()
         logger.critical("Switching to CRITICAL threshold")
 
     @expose_command()
@@ -1277,7 +1294,11 @@ class Qtile(CommandObject):
 
     @expose_command()
     def spawn(
-        self, cmd: list[str] | str, shell: bool = False, env: dict[str, str] = dict()
+        self,
+        cmd: list[str] | str,
+        shell: bool = False,
+        env: dict[str, str] = dict(),
+        group: str | None = None,
     ) -> int:
         """
         Spawn a new process.
@@ -1291,6 +1312,8 @@ class Qtile(CommandObject):
             -c". This enables the use of shell syntax within the command (e.g. pipes).
         env:
             Dictionary of environmental variables to pass with command.
+        group:
+            Name of group in which to spawn process.
 
         Examples
         ========
@@ -1326,20 +1349,58 @@ class Qtile(CommandObject):
         # std{in,out,err} should be /dev/null
         with open("/dev/null") as null:
             file_actions: list[tuple] = [
-                (os.POSIX_SPAWN_DUP2, 0, null.fileno()),
+                (os.POSIX_SPAWN_DUP2, null.fileno(), 0),
                 (os.POSIX_SPAWN_DUP2, 1, null.fileno()),
                 (os.POSIX_SPAWN_DUP2, 2, null.fileno()),
             ]
 
-            if sys.version_info.major >= 3 and sys.version_info.minor >= 13:
+            # To create a rule to move process to a specific group and manage race conditions
+            # we need to do the following:
+            # 1) Create a socket pair so we can communicate to child process
+            # 2) Spawn a "qtile launch" process and pass the file descriptor
+            # 3) Have "qtile launch" notify qtile that it's ready
+            # 4) Create a rule using the pid from step 2
+            # 5) Send a message over the socket to "qtile launch"
+            # 6) Once the message is received, "qtile launch" execs the desired process
+            if group is not None:
+                parent_sock, child_sock = socket.socketpair()
+                # socket pair fds are non-inheritable by default but we need this to survice
+                os.set_inheritable(child_sock.fileno(), True)
+
+            # this API is only available on python 3.13 or above, and only on platforms
+            # where posix_spawn_file_actions_addclosefrom_np() exists (only glibc on Linux).
+            # where we're creating a rule to manage the group, we'll handle closing fds in
+            # `qtile launch`
+            if hasattr(os, "POSIX_SPAWN_CLOSEFROM") and group is None:
                 # we should close all fds so that child processes don't
                 # accidentally write to our x11 event loop or whatever; we never
-                # used to do this, so it seems fine to only do it on python 3.13 or
-                # above, where this nice API to do it exists.
-                file_actions.append((os.POSIX_SPAWN_CLOSEFROM, 3))  # type: ignore
+                # used to do this, so it seems fine to only do it where this nice
+                # API to do it exists.
+                file_actions.append((os.POSIX_SPAWN_CLOSEFROM, 3))
 
             try:
-                return os.posix_spawnp(args[0], args, env, file_actions=file_actions)
+                if group is not None:
+                    # As per step 2 (above) we want to launch the process via "qtile launch"
+                    args = ["qtile", "launch", "--fd", f"{child_sock.fileno()}"] + args
+                pid = os.posix_spawnp(args[0], args, env, file_actions=file_actions)
+                if group is not None:
+                    # Listen for READY message from `qtile launch`
+                    data = parent_sock.recv(5)
+
+                    if data != b"READY":
+                        logger.warning("Received unexpected data: {data!r}")
+
+                    # Create the group matching rule
+                    match_args = {"net_wm_pid": pid}
+                    rule_args = {"group": group, "one_time": True}
+                    self.add_rule(match_args=match_args, rule_args=rule_args)
+
+                    # Tell child process it can now spawn main proces as rule has been added
+                    parent_sock.send(b"OK")
+
+                    parent_sock.close()
+                    child_sock.close()
+                return pid
             except OSError as e:
                 logger.warning("failed to execute: %s: %s", str(args), str(e))
                 return -1
@@ -1819,3 +1880,14 @@ class Qtile(CommandObject):
     def fire_user_hook(self, hook_name: str, *args: Any) -> None:
         """Fire a custom hook."""
         hook.fire(f"user_{hook_name}", *args)
+
+    @expose_command()
+    def start_repl_server(self, locals_dict: dict[str, Any] = dict()) -> None:
+        """Start the REPL server."""
+        _locals = {"qtile": self, **locals_dict}
+        create_task(repl_server.start(locals_dict=_locals))
+
+    @expose_command()
+    def stop_repl_server(self) -> None:
+        """Stop the REPL server."""
+        create_task(repl_server.stop())
