@@ -1,38 +1,16 @@
-# -*- coding:utf-8 -*-
-# Copyright (c) 2011 Florian Mounier
-# Copyright (c) 2012-2013, 2015 Tycho Andersen
-# Copyright (c) 2013 Björn Lindström
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2014 ramnes
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2014 dmpayton
-# Copyright (c) 2014 dequis
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
-from __future__ import division
+from __future__ import annotations
 
 import math
+from typing import TYPE_CHECKING
 
-from .base import Layout
-from .. import utils
+from libqtile.command.base import expose_command
+from libqtile.layout.base import _SimpleLayoutBase
 
+if TYPE_CHECKING:
+    from typing import Any, Self
+
+    from libqtile.backend.base import Window
+    from libqtile.group import _Group
 
 ROWCOL = 1  # do rows at a time left to right top down
 COLROW = 2  # do cols top to bottom, left to right
@@ -40,7 +18,7 @@ COLROW = 2  # do cols top to bottom, left to right
 GOLDEN_RATIO = 1.618
 
 
-class GridInfo(object):
+class GridInfo:
     """
     Calculates sizes for grids
     >>> gi = GridInfo(.5, 5, 600, 480)
@@ -60,6 +38,7 @@ class GridInfo(object):
 
 
     """
+
     def __init__(self, ratio, num_windows, width, height):
         self.ratio = ratio
         self.num_windows = num_windows
@@ -73,7 +52,6 @@ class GridInfo(object):
         best_ratio = None
         best_rows_cols_orientation = None
         for rows, cols, orientation in self._possible_grids(num_windows):
-
             sample_width = width / cols
             sample_height = height / rows
             sample_ratio = sample_width / sample_height
@@ -99,17 +77,14 @@ class GridInfo(object):
                 # also want the reverse test
                 yield (cols, rows, COLROW)
 
-    def get_sizes_advanced(self, total_width, total_height,
-                           xoffset=0, yoffset=0):
+    def get_sizes_advanced(self, total_width, total_height, xoffset=0, yoffset=0):
         """after every row/column recalculate remaining area"""
         results = []
         width = total_width
         height = total_height
         while len(results) < self.num_windows:
             remaining = self.num_windows - len(results)
-            orien, sizes = self._get_row_or_col(
-                remaining, width, height, xoffset, yoffset
-            )
+            orien, sizes = self._get_row_or_col(remaining, width, height, xoffset, yoffset)
             results.extend(sizes)
             if orien == ROWCOL:
                 # adjust height/yoffset
@@ -151,9 +126,7 @@ class GridInfo(object):
         width = 0
         height = 0
         results = []
-        rows, cols, orientation = self.calc(
-            self.num_windows, total_width, total_height
-        )
+        rows, cols, orientation = self.calc(self.num_windows, total_width, total_height)
         if orientation == ROWCOL:
             y = 0
             for i, row in enumerate(range(rows)):
@@ -170,12 +143,7 @@ class GridInfo(object):
                         # make last column (or item) take up remaining space
                         width = total_width - x
 
-                    results.append((
-                        x + xoffset,
-                        y + yoffset,
-                        width,
-                        height
-                    ))
+                    results.append((x + xoffset, y + yoffset, width, height))
                     if len(results) == self.num_windows:
                         return results
                     x += width
@@ -193,12 +161,14 @@ class GridInfo(object):
                         height = total_height // remaining
                     elif j == rows - 1 or len(results) + 1 == self.num_windows:
                         height = total_height - y
-                    results.append((
-                        x + xoffset,  # i * width + xoffset,
-                        y + yoffset,  # j * height + yoffset,
-                        width,
-                        height
-                    ))
+                    results.append(
+                        (
+                            x + xoffset,  # i * width + xoffset,
+                            y + yoffset,  # j * height + yoffset,
+                            width,
+                            height,
+                        )
+                    )
                     if len(results) == self.num_windows:
                         return results
                     y += height
@@ -207,52 +177,37 @@ class GridInfo(object):
         return results
 
 
-class RatioTile(Layout):
+class RatioTile(_SimpleLayoutBase):
     """Tries to tile all windows in the width/height ratio passed in"""
+
     defaults = [
-        ("border_focus", "#0000ff", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused windows."),
+        ("border_focus", "#0000ff", "Border colour(s) for the focused window."),
+        ("border_normal", "#000000", "Border colour(s) for un-focused windows."),
         ("border_width", 1, "Border width."),
-        ("name", "ratiotile", "Name of this layout."),
-        ("margin", 0, "Margin of the layout"),
+        ("margin", 0, "Margin of the layout (int or list of ints [N E S W])"),
         ("ratio", GOLDEN_RATIO, "Ratio of the tiles"),
         ("ratio_increment", 0.1, "Amount to increment per ratio increment"),
         ("fancy", False, "Use a different method to calculate window sizes."),
     ]
 
     def __init__(self, **config):
-        Layout.__init__(self, **config)
+        _SimpleLayoutBase.__init__(self, **config)
         self.add_defaults(RatioTile.defaults)
-        self.clients = []
-        self.focused = None
         self.dirty = True  # need to recalculate
         self.layout_info = []
         self.last_size = None
         self.last_screen = None
 
-    def clone(self, group):
-        c = Layout.clone(self, group)
-        c.clients = []
-        return c
+    def clone(self, group: _Group) -> Self:
+        return _SimpleLayoutBase.clone(self, group)
 
-    def focus(self, c):
-        self.focused = c
-
-    def blur(self):
-        self.focused = None
-
-    def add(self, w):
+    def add_client(self, w: Window) -> None:  # type: ignore[override]
         self.dirty = True
-        self.clients.insert(0, w)
+        self.clients.append_head(w)
 
-    def remove(self, w):
+    def remove(self, w: Window) -> Window | None:
         self.dirty = True
-        if self.focused is w:
-            self.focused = None
-        self.clients.remove(w)
-        if self.clients:  # and w is self.focused:
-            self.focused = self.clients[0]
-        return self.focused
+        return _SimpleLayoutBase.remove(self, w)
 
     def configure(self, win, screen):
         # force recalc
@@ -260,28 +215,17 @@ class RatioTile(Layout):
             self.last_screen = screen
             self.dirty = True
         if self.last_size and not self.dirty:
-            if screen.width != self.last_size[0] or \
-                    screen.height != self.last_size[1]:
+            if screen.width != self.last_size[0] or screen.height != self.last_size[1]:
                 self.dirty = True
         if self.dirty:
-            gi = GridInfo(
-                self.ratio,
-                len(self.clients),
-                screen.width,
-                screen.height
-            )
+            gi = GridInfo(self.ratio, len(self.clients), screen.width, screen.height)
             self.last_size = (screen.width, screen.height)
             if self.fancy:
                 method = gi.get_sizes_advanced
             else:
                 method = gi.get_sizes
 
-            self.layout_info = method(
-                screen.width,
-                screen.height,
-                screen.x,
-                screen.y
-            )
+            self.layout_info = method(screen.width, screen.height, screen.x, screen.y)
 
             self.dirty = False
         try:
@@ -291,9 +235,9 @@ class RatioTile(Layout):
             return
         x, y, w, h = self.layout_info[idx]
         if win.has_focus:
-            bc = self.group.qtile.colorPixel(self.border_focus)
+            bc = self.border_focus
         else:
-            bc = self.group.qtile.colorPixel(self.border_normal)
+            bc = self.border_normal
         win.place(
             x,
             y,
@@ -305,100 +249,46 @@ class RatioTile(Layout):
         )
         win.unhide()
 
-    def info(self):
-        return {
-            'clients': [x.name for x in self.clients],
-            'ratio': self.ratio,
-            'focused': self.focused.name if self.focused else None,
-            'layout_info': self.layout_info
-        }
+    @expose_command()
+    def info(self) -> dict[str, Any]:
+        d = _SimpleLayoutBase.info(self)
+        focused = self.clients.current_client
+        d["ratio"] = self.ratio
+        d["focused"] = focused.name if focused else None
+        d["layout_info"] = self.layout_info
+        return d
 
-    def shuffleUp(self):
+    @expose_command("down")
+    def previous(self) -> None:
+        _SimpleLayoutBase.previous(self)
+
+    @expose_command("up")
+    def next(self) -> None:
+        _SimpleLayoutBase.next(self)
+
+    @expose_command()
+    def shuffle_down(self):
         if self.clients:
-            utils.shuffleUp(self.clients)
-            self.group.layoutAll()
+            self.clients.rotate_up()
+            self.group.layout_all()
 
-    def shuffleDown(self):
+    @expose_command()
+    def shuffle_up(self):
         if self.clients:
-            utils.shuffleDown(self.clients)
-            self.group.layoutAll()
+            self.clients.rotate_down()
+            self.group.layout_all()
 
-    def focus_first(self):
-        if self.clients:
-            return self.clients[0]
-
-    def focus_next(self, win):
-        idx = self.clients.index(win)
-        if len(self.clients) > idx + 1:
-            return self.clients[idx + 1]
-
-    def focus_last(self):
-        if self.clients:
-            return self.clients[-1]
-
-    def focus_previous(self, win):
-        idx = self.clients.index(win)
-        if idx > 0:
-            return self.clients[idx - 1]
-
-    def getNextClient(self):
-        previndex = self.clients.index(self.focused) - 1
-        if previndex < 0:
-            previndex = len(self.clients) - 1
-        return self.clients[previndex]
-
-    def getPreviousClient(self):
-        nextindex = self.clients.index(self.focused) + 1
-        if nextindex >= len(self.clients):
-            nextindex = 0
-        return self.clients[nextindex]
-
-    def next(self):
-        n = self.getPreviousClient()
-        self.group.focus(n)
-
-    def previous(self):
-        n = self.getNextClient()
-        self.group.focus(n)
-
-    def shuffle(self, function):
-        if self.clients:
-            function(self.clients)
-            self.group.layoutAll()
-
-    def cmd_down(self):
-        self.previous()
-
-    def cmd_up(self):
-        self.next()
-
-    def cmd_next(self):
-        self.next()
-
-    def cmd_previous(self):
-        self.previous()
-
-    def cmd_shuffle_down(self):
-        self.shuffleDown()
-
-    def cmd_shuffle_up(self):
-        self.shuffleUp()
-
-    def cmd_decrease_ratio(self):
+    @expose_command()
+    def decrease_ratio(self):
         new_ratio = self.ratio - self.ratio_increment
         if new_ratio < 0:
             return
         self.ratio = new_ratio
-        self.group.layoutAll()
+        self.dirty = True
+        self.group.layout_all()
 
-    def cmd_increase_ratio(self):
+    @expose_command()
+    def increase_ratio(self):
         self.ratio += self.ratio_increment
-        self.group.layoutAll()
-
-    def cmd_info(self):
-        return self.info()
-
-
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
+        self.dirty = True
+        self.group.layout_all()
