@@ -1,32 +1,3 @@
-# Copyright (c) 2011 Mounier Florian
-# Copyright (c) 2011 Paul Colomiets
-# Copyright (c) 2012 roger
-# Copyright (c) 2012-2014 Tycho Andersen
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2013 Arnas Udovicius
-# Copyright (c) 2014 ramnes
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2014 Nathan Hoad
-# Copyright (c) 2014 dequis
-# Copyright (c) 2014 Thomas Sarboni
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
@@ -362,8 +333,6 @@ class TreeTab(Layout):
         ("urgent_fg", "ffffff", "Foreground color of urgent tab"),
         ("inactive_bg", "606060", "Background color of inactive tab"),
         ("inactive_fg", "ffffff", "Foreground color of inactive tab"),
-        ("margin_left", 6, "Left margin of tab panel"),
-        ("margin_y", 6, "Vertical margin of tab panel"),
         ("padding_left", 6, "Left padding for tabs"),
         ("padding_x", 6, "Left padding for tab label"),
         ("padding_y", 2, "Top padding for tab label"),
