@@ -1,22 +1,3 @@
-# Copyright (c) 2023 elParaguayo
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
 import asyncio
 import contextlib
 from enum import Enum
@@ -70,6 +51,10 @@ class _BluetoothBase:
         self.properties.on_properties_changed(self.properties_changed)
         self._name = ""
 
+    @property
+    def name(self):
+        return self._name
+
     def __repr__(self):
         """Neater repr to help debugging."""
         return f"<{self.__class__.__name__}: {self.name} ({self.path})>"
@@ -127,10 +112,6 @@ class BluetoothDevice(_BluetoothBase):
         elif not self.paired:
             await self.pair_and_connect()
 
-    @property
-    def name(self):
-        return self._name
-
     @property
     def connected(self):
         return self._connected
@@ -258,10 +239,6 @@ class BluetoothAdapter(_BluetoothBase):
     def powered(self):
         return self._powered
 
-    @property
-    def name(self):
-        return self._name
-
     async def discover(self):
         if self.discovering:
             await self.stop_discovery()
@@ -277,7 +254,7 @@ class BluetoothAdapter(_BluetoothBase):
             self.widget.refresh()
 
 
-class Bluetooth(base._TextBox, base.MarginMixin):
+class Bluetooth(base._TextBox):
     """
     Bluetooth widget that provides following functionality:
     - View multiple adapters/devices (adapters can be filtered)
@@ -364,7 +341,6 @@ class Bluetooth(base._TextBox, base.MarginMixin):
     def __init__(self, **config):
         base._TextBox.__init__(self, **config)
         self.add_defaults(Bluetooth.defaults)
-        self.add_defaults(base.MarginMixin.defaults)
         self.connected = False
         self.bus = None
         self.devices = {}
