@@ -1,9 +1,13 @@
 import json
-from typing import Any, List, Tuple
-from urllib.error import URLError
-from urllib.request import Request, urlopen
+from typing import Any
+
+try:
+    import aiohttp
+except ImportError:
+    aiohttp = None
 
 from libqtile.log_utils import logger
+from libqtile.utils import acall_process
 from libqtile.widget import base
 
 try:
@@ -19,7 +23,7 @@ except ImportError:
         raise Exception("no xmltodict library")
 
 
-class GenPollText(base.ThreadPoolText):
+class GenPollText(base.BackgroundPoll):
     """A generic text widget that polls using poll function to get the text"""
 
     defaults = [
@@ -27,7 +31,7 @@ class GenPollText(base.ThreadPoolText):
     ]
 
     def __init__(self, **config):
-        base.ThreadPoolText.__init__(self, "", **config)
+        base.BackgroundPoll.__init__(self, "", **config)
         self.add_defaults(GenPollText.defaults)
 
     def poll(self):
@@ -36,10 +40,10 @@ class GenPollText(base.ThreadPoolText):
         return self.func()
 
 
-class GenPollUrl(base.ThreadPoolText):
+class GenPollUrl(base.BackgroundPoll):
     """A generic text widget that polls an url and parses it using parse function"""
 
-    defaults = [
+    defaults: list[tuple[str, Any, str]] = [
         ("url", None, "Url"),
         ("data", None, "Post Data"),
         ("parse", None, "Parse Function"),
@@ -47,10 +51,10 @@ class GenPollUrl(base.ThreadPoolText):
         ("user_agent", "Qtile", "Set the user agent"),
         ("headers", {}, "Extra Headers"),
         ("xml", False, "Is XML?"),
-    ]  # type: List[Tuple[str, Any, str]]
+    ]
 
     def __init__(self, **config):
-        base.ThreadPoolText.__init__(self, "", **config)
+        base.BackgroundPoll.__init__(self, "", **config)
         self.add_defaults(GenPollUrl.defaults)
 
         self.headers["User-agent"] = self.user_agent
@@ -60,35 +64,55 @@ class GenPollUrl(base.ThreadPoolText):
         if self.data and not isinstance(self.data, str):
             self.data = json.dumps(self.data).encode()
 
-    def fetch(self):
-        req = Request(self.url, self.data, self.headers)
-        res = urlopen(req)
-        charset = res.headers.get_content_charset()
-
-        body = res.read()
-        if charset:
-            body = body.decode(charset)
-
-        if self.json:
-            body = json.loads(body)
-
-        if self.xml:
-            body = xmlparse(body)
-        return body
-
-    def poll(self):
+    async def apoll(self):
         if not self.parse or not self.url:
             return "Invalid config"
 
-        try:
-            body = self.fetch()
-        except URLError:
-            return "No network"
+        headers = self.headers.copy()
+        data = self.data
 
         try:
+            async with aiohttp.ClientSession() as session:
+                async with session.request(
+                    method="POST" if data else "GET", url=self.url, data=data, headers=headers
+                ) as response:
+                    if self.json:
+                        body = await response.json()
+                    elif self.xml:
+                        text_body = await response.text()
+                        body = xmlparse(text_body)
+                    else:
+                        body = await response.text()
+
             text = self.parse(body)
         except Exception:
             logger.exception("got exception polling widget")
             text = "Can't parse"
 
         return text
+
+
+class GenPollCommand(base.BackgroundPoll):
+    """A generic text widget to display output from scripts or shell commands"""
+
+    defaults = [
+        ("update_interval", 60, "update time in seconds"),
+        ("cmd", None, "command line as a string or list of arguments to execute"),
+        ("shell", False, "run command through shell to enable piping and shell expansion"),
+        ("parse", None, "Function to parse output of command"),
+    ]
+
+    def __init__(self, **config):
+        base.BackgroundPoll.__init__(self, "", **config)
+        self.add_defaults(GenPollCommand.defaults)
+
+    def _configure(self, qtile, bar):
+        base.BackgroundPoll._configure(self, qtile, bar)
+        self.add_callbacks({"Button1": self.force_update})
+
+    async def apoll(self):
+        out = await acall_process(self.cmd, self.shell)
+        if self.parse:
+            return self.parse(out)
+
+        return out.strip()
