@@ -1,36 +1,3 @@
-# Copyright (c) 2010 Aldo Cortesi
-# Copyright (c) 2010-2011 Paul Colomiets
-# Copyright (c) 2010, 2014 roger
-# Copyright (c) 2011 Mounier Florian
-# Copyright (c) 2011 Kenji_Takahashi
-# Copyright (c) 2012 Mika Fischer
-# Copyright (c) 2012, 2014-2015 Tycho Andersen
-# Copyright (c) 2012-2013 Craig Barnes
-# Copyright (c) 2013 dequis
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2013 Mickael FALCK
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2014 Adi Sieker
-# Copyright (c) 2014 Florian Scherf
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 import itertools
 import operator
 import time
@@ -52,15 +19,13 @@ __all__ = [
 ]
 
 
-class _Graph(base._Widget):
+class _Graph(base._Widget, base.MarginMixin):
     fixed_upper_bound = False
     defaults = [
         ("graph_color", "18BAEB", "Graph color"),
         ("fill_color", "1667EB.3", "Fill color for linefill graph"),
         ("border_color", "215578", "Widget border color"),
         ("border_width", 2, "Widget border width"),
-        ("margin_x", 3, "Margin X"),
-        ("margin_y", 3, "Margin Y"),
         ("samples", 100, "Count of graph samples."),
         ("frequency", 1, "Update frequency in seconds"),
         ("type", "linefill", "'box', 'line', 'linefill'"),
@@ -86,11 +51,11 @@ class _Graph(base._Widget):
 
     @property
     def graphwidth(self):
-        return self.width - self.border_width * 2 - self.margin_x * 2
+        return self.width - self.margin_x * 2 - self.border_width * 2
 
     @property
     def graphheight(self):
-        return self.bar.height - self.margin_y * 2 - self.border_width * 2
+        return self.height - self.margin_y * 2 - self.border_width * 2
 
     def step(self):
         return self.graphwidth / float(self.samples)
@@ -155,7 +120,7 @@ class _Graph(base._Widget):
                 self.margin_x + self.border_width / 2.0,
                 self.margin_y + self.border_width / 2.0,
                 self.graphwidth + self.border_width,
-                self.bar.height - self.margin_y * 2 - self.border_width,
+                self.graphheight - self.border_width,
             )
             self.drawer.ctx.stroke()
         x = self.margin_x + self.border_width
@@ -176,7 +141,7 @@ class _Graph(base._Widget):
         else:
             raise ValueError(f"Unknown graph type: {self.type}.")
 
-        self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.width)
+        self.draw_at_default_position()
 
     def push(self, value):
         if self.lag_cycles > self.samples:
