@@ -1,22 +1,4 @@
-# Copyright (c) 2021 elParaguayo
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
+from asyncio import current_task
 from collections.abc import Callable
 from contextlib import suppress
 from functools import partial
@@ -132,6 +114,7 @@ class StatusNotifierItem:  # noqa: E303
         self.icon_theme = icon_theme
         self.icon = None
         self.path = path if path else STATUSNOTIFIER_PATH
+        self.last_icon_name = None
 
     def __eq__(self, other):
         # Convenience method to find Item in list by service path
@@ -248,16 +231,28 @@ class StatusNotifierItem:  # noqa: E303
         # the app. We also don't want to do the recursive lookup with an empty
         # icon name as, otherwise, the glob will match things that are not images.
         if icon_name:
+            if icon_name == self.last_icon_name:
+                current_task().remove_done_callback(self._redraw)
+                return
+
+            self.last_icon_name = icon_name
+            self.icon = None
+
             try:
                 icon_path = await self.item.get_icon_theme_path()
             except (AttributeError, DBusError):
                 icon_path = None
 
+            icon = None
             if icon_path:
-                self.icon = self._get_custom_icon(icon_name, Path(icon_path))
+                icon = self._get_custom_icon(icon_name, Path(icon_path))
 
-            if not self.icon:
+            if icon:
+                self.icon = icon
+            else:
                 self.icon = self._get_xdg_icon(icon_name)
+        else:
+            self.icon = None
 
         if fallback:
             for icon in ["Icon", "Attention", "Overlay"]:
@@ -274,7 +269,6 @@ class StatusNotifierItem:  # noqa: E303
         task.add_done_callback(self._redraw)
 
     def _update_local_icon(self):
-        self.icon = None
         self._create_task_and_draw(self._get_local_icon())
 
     def _new_icon(self):
