@@ -1,28 +1,3 @@
-# Copyright (c) 2011 Timo Schmiade
-# Copyright (c) 2012 Phil Jackson
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2014 Tycho Andersen
-# Copyright (c) 2016 Christoph Lassner
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 from __future__ import annotations
 
 import mailbox
@@ -31,7 +6,7 @@ import os.path
 from libqtile.widget import base
 
 
-class Maildir(base.ThreadPoolText):
+class Maildir(base.BackgroundPoll):
     """A simple widget showing the number of new mails in maildir mailboxes"""
 
     defaults = [
@@ -39,7 +14,7 @@ class Maildir(base.ThreadPoolText):
         (
             "sub_folders",
             [{"path": "INBOX", "label": "Home mail"}, {"path": "spam", "label": "Home junk"}],
-            "List of subfolders to scan. Each subfolder is a dict of `path` " "and `label`.",
+            "List of subfolders to scan. Each subfolder is a dict of `path` and `label`.",
         ),
         ("separator", " ", "the string to put between the subfolder strings."),
         (
@@ -51,7 +26,7 @@ class Maildir(base.ThreadPoolText):
         (
             "hide_when_empty",
             False,
-            "Whether not to display anything if " "the subfolder has no new mail",
+            "Whether not to display anything if the subfolder has no new mail",
         ),
         ("empty_color", None, "Display color when no new mail is available"),
         ("nonempty_color", None, "Display color when new mail is available"),
@@ -59,7 +34,7 @@ class Maildir(base.ThreadPoolText):
     ]
 
     def __init__(self, **config):
-        base.ThreadPoolText.__init__(self, "", **config)
+        base.BackgroundPoll.__init__(self, "", **config)
         self.add_defaults(Maildir.defaults)
 
         # if it looks like a list of strings then we just convert them
