@@ -80,7 +80,7 @@ format_fns = {
 }
 
 
-class Mpd2(base.ThreadPoolText):
+class Mpd2(base.BackgroundPoll):
     r"""Mpd2 Object.
 
     Parameters
@@ -237,7 +237,7 @@ class Mpd2(base.ThreadPoolText):
 
     def button_press(self, x, y, button):
         """handle click event on widget."""
-        base.ThreadPoolText.button_press(self, x, y, button)
+        base.BackgroundPoll.button_press(self, x, y, button)
         m_name = self.mouse_buttons[button]
 
         if self.connected:
