@@ -1,34 +1,9 @@
-# Copyright (c) 2012 TiN
-# Copyright (c) 2012, 2014 Tycho Andersen
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2014-2015 Sean Vig
-# Copyright (c) 2014 Adi Sieker
-# Copyright (c) 2014 Foster McLane
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 import psutil
 
 from libqtile.widget import base
 
 
-class ThermalSensor(base.ThreadPoolText):
+class ThermalSensor(base.InLoopPollText):
     """Widget to display temperature sensor information
 
     For using the thermal sensor widget you need to have lm-sensors installed.
@@ -62,7 +37,7 @@ class ThermalSensor(base.ThreadPoolText):
     ]
 
     def __init__(self, **config):
-        base.ThreadPoolText.__init__(self, **config)
+        base.InLoopPollText.__init__(self, **config)
         self.add_defaults(ThermalSensor.defaults)
         temp_values = self.get_temp_sensors()
 
@@ -77,7 +52,7 @@ class ThermalSensor(base.ThreadPoolText):
 
     def _configure(self, qtile, bar):
         self.unit = "°C" if self.metric else "°F"
-        base.ThreadPoolText._configure(self, qtile, bar)
+        base.InLoopPollText._configure(self, qtile, bar)
         self.foreground_normal = self.foreground
 
     def get_temp_sensors(self):
