@@ -1,41 +1,11 @@
-# Copyright (c) 2010, 2012, 2014 roger
-# Copyright (c) 2011 Kirk Strauser
-# Copyright (c) 2011 Florian Mounier
-# Copyright (c) 2011 Mounier Florian
-# Copyright (c) 2011 Roger Duran
-# Copyright (c) 2012-2015 Tycho Andersen
-# Copyright (c) 2013 Tao Sauvage
-# Copyright (c) 2013 Craig Barnes
-# Copyright (c) 2014-2015 Sean Vig
-# Copyright (c) 2014 Adi Sieker
-# Copyright (c) 2014 dmpayton
-# Copyright (c) 2014 Jody Frankowski
-# Copyright (c) 2016 Christoph Lassner
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
+import asyncio
 import re
 import subprocess
 
 from libqtile import bar
 from libqtile.command.base import expose_command
 from libqtile.log_utils import logger
+from libqtile.utils import acall_process, create_task
 from libqtile.widget import base
 
 __all__ = [
@@ -105,7 +75,7 @@ class Volume(base._TextBox):
     ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, "0", **config)
+        base._TextBox.__init__(self, "", **config)
         self.add_defaults(Volume.defaults)
         self.surfaces = {}
         self.volume = None
@@ -120,6 +90,11 @@ class Volume(base._TextBox):
             }
         )
 
+    def timer_setup(self):
+        create_task(self.do_volume())
+        if self.theme_path:
+            self.setup_images()
+
     def _configure(self, qtile, parent_bar):
         if self.theme_path:
             self.length_type = bar.STATIC
@@ -127,12 +102,7 @@ class Volume(base._TextBox):
         base._TextBox._configure(self, qtile, parent_bar)
         self.unmute_foreground = self.foreground
 
-    def timer_setup(self):
-        self.timeout_add(self.update_interval, self.update)
-        if self.theme_path:
-            self.setup_images()
-
-    def create_amixer_command(self, *args):
+    def create_amixer_command(self, *args) -> str:
         cmd = ["amixer"]
 
         if self.cardid is not None:
@@ -148,8 +118,8 @@ class Volume(base._TextBox):
         base._TextBox.button_press(self, x, y, button)
         self.draw()
 
-    def update(self):
-        vol, muted = self.get_volume()
+    async def do_volume(self):
+        vol, muted = await self.get_volume()
         if vol != self.volume or muted != self.is_mute:
             self.volume = vol
             self.is_mute = muted
@@ -157,11 +127,12 @@ class Volume(base._TextBox):
             # to figure out how big it is and draw it.
             self._update_drawer()
             self.bar.draw()
-        self.timeout_add(self.update_interval, self.update)
+        await asyncio.sleep(self.update_interval)
+        create_task(self.do_volume())
 
     def _update_drawer(self):
         if self.mute_foreground is not None:
-            self.foreground = self.mute_foreground if self.is_mute else self.unmute_foreground
+            self.layout.colour = self.mute_foreground if self.is_mute else self.unmute_foreground
 
         if self.theme_path:
             self.drawer.clear(self.background or self.bar.background)
@@ -206,27 +177,27 @@ class Volume(base._TextBox):
             "audio-volume-muted",
         )
         d_images = images.Loader(self.theme_path)(*names)
+        new_height = self.bar.size - 2
         for name, img in d_images.items():
-            new_height = self.bar.height - 1
             img.resize(height=new_height)
             if img.width > self.length:
-                self.length = img.width + self.actual_padding * 2
+                self.length = img.width + self.padding * 2
             self.surfaces[name] = img.pattern
 
-    def get_volume(self):
+    async def get_volume(self):
         try:
             if self.get_volume_command is not None:
                 get_volume_cmd = self.get_volume_command
             else:
                 get_volume_cmd = self.create_amixer_command("sget", self.channel)
 
-            mixer_out = subprocess.getoutput(get_volume_cmd)
+            mixer_out = await acall_process(get_volume_cmd, shell=True)
         except subprocess.CalledProcessError:
             return -1, False
 
         check_mute = mixer_out
         if self.check_mute_command:
-            check_mute = subprocess.getoutput(self.check_mute_command)
+            check_mute = await acall_process(self.check_mute_command, shell=True)
 
         muted = self.check_mute_string in check_mute
 
@@ -239,7 +210,7 @@ class Volume(base._TextBox):
 
     def draw(self):
         if self.theme_path:
-            self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.length)
+            self.draw_at_default_position()
         else:
             base._TextBox.draw(self)
 
