@@ -1,14 +1,17 @@
 import os
+import shutil
 import subprocess
 import tempfile
+from multiprocessing import Value
 
 import pytest
 import xcffib.xproto
 import xcffib.xtest
 
 import libqtile.config
-from libqtile import layout, utils
+from libqtile import hook, layout, utils
 from libqtile.backend.x11 import window, xcbq
+from libqtile.backend.x11.xcbq import Connection
 from test.conftest import dualmonitor
 from test.helpers import (
     HEIGHT,
@@ -60,6 +63,90 @@ def test_change_state_via_message(xmanager, conn):
     assert not xmanager.c.window.info()["minimized"]
 
 
+def set_urgent(w):
+    w.urgent = True
+    hook.fire("client_urgent_hint_changed", w)
+    return False
+
+
+class UrgentConfig(BareConfig):
+    focus_on_window_activation = "urgent"
+
+
+class SmartConfig(BareConfig):
+    focus_on_window_activation = "smart"
+
+
+class FuncConfig(BareConfig):
+    # must be a static method here because otherwise it gets turned into a MethodType (we need a FunctionType)
+    # this is only an issue in this test and not the real config file
+    focus_on_window_activation = staticmethod(set_urgent)
+
+
+@dualmonitor
+def test_urgent_hook_fire(xmanager_nospawn):
+    xmanager_nospawn.display = xmanager_nospawn.backend.env["DISPLAY"]
+    conn = Connection(xmanager_nospawn.display)
+
+    xmanager_nospawn.hook_fired = Value("i", 0)
+
+    def _hook_test(val):
+        xmanager_nospawn.hook_fired.value += 1
+
+    hook.subscribe.client_urgent_hint_changed(_hook_test)
+
+    xmanager_nospawn.start(UrgentConfig)
+
+    xmanager_nospawn.test_window("one")
+    window_info = xmanager_nospawn.c.window.info()
+
+    # send activate window message
+    data = xcffib.xproto.ClientMessageData.synthetic([0, 0, 0, 0, 0], "IIIII")
+    ev = xcffib.xproto.ClientMessageEvent.synthetic(
+        32, window_info["id"], conn.atoms["_NET_ACTIVE_WINDOW"], data
+    )
+    conn.default_screen.root.send_event(ev, mask=xcffib.xproto.EventMask.SubstructureRedirect)
+    conn.xsync()
+
+    xmanager_nospawn.terminate()
+    assert xmanager_nospawn.hook_fired.value == 1
+
+    # test that focus_on_window_activation = "smart" also fires the hook
+    xmanager_nospawn.start(SmartConfig, no_spawn=True)
+
+    xmanager_nospawn.test_window("one")
+    window_info = xmanager_nospawn.c.window.info()
+    xmanager_nospawn.c.window.toscreen(1)
+
+    # send activate window message
+    ev = xcffib.xproto.ClientMessageEvent.synthetic(
+        32, window_info["id"], conn.atoms["_NET_ACTIVE_WINDOW"], data
+    )
+    conn.default_screen.root.send_event(ev, mask=xcffib.xproto.EventMask.SubstructureRedirect)
+    conn.xsync()
+    xmanager_nospawn.terminate()
+
+    assert xmanager_nospawn.hook_fired.value == 2
+
+    # test that a custom function also fires the hook
+    xmanager_nospawn.start(FuncConfig, no_spawn=True)
+
+    xmanager_nospawn.test_window("one")
+    window_info = xmanager_nospawn.c.window.info()
+    xmanager_nospawn.c.window.toscreen(1)
+
+    # send activate window message
+    ev = xcffib.xproto.ClientMessageEvent.synthetic(
+        32, window_info["id"], conn.atoms["_NET_ACTIVE_WINDOW"], data
+    )
+    conn.default_screen.root.send_event(ev, mask=xcffib.xproto.EventMask.SubstructureRedirect)
+    conn.xsync()
+
+    xmanager_nospawn.terminate()
+
+    assert xmanager_nospawn.hook_fired.value == 3
+
+
 @manager_config
 def test_default_float_hints(xmanager, conn):
     xmanager.c.next_layout()
@@ -317,7 +404,7 @@ def test_hints_setting_unsetting(xmanager, conn):
         # We default the input hint to true since some non-trivial number of
         # windows don't set it, and most of them want focus. The spec allows
         # WMs to assume "convenient" values.
-        assert xmanager.c.window.hints()["input"]
+        assert xmanager.c.window.get_hints()["input"]
 
         # now try to "update" it, but don't really set an update (i.e. the
         # InputHint bit is 0, so the WM should not derive a new hint from the
@@ -327,21 +414,21 @@ def test_hints_setting_unsetting(xmanager, conn):
         conn.flush()
 
         # should still have the hint
-        assert xmanager.c.window.hints()["input"]
+        assert xmanager.c.window.get_hints()["input"]
 
         # now do an update: turn it off
         hints[0] = xcbq.HintsFlags["InputHint"]
         hints[1] = 0
         w.set_property("WM_HINTS", hints, type="WM_HINTS", format=32)
         conn.flush()
-        assert not xmanager.c.window.hints()["input"]
+        assert not xmanager.c.window.get_hints()["input"]
 
         # turn it back on
         hints[0] = xcbq.HintsFlags["InputHint"]
         hints[1] = 1
         w.set_property("WM_HINTS", hints, type="WM_HINTS", format=32)
         conn.flush()
-        assert xmanager.c.window.hints()["input"]
+        assert xmanager.c.window.get_hints()["input"]
 
     finally:
         w.kill_client()
@@ -689,10 +776,20 @@ def test_multiple_borders(xmanager):
     wid = xmanager.c.window.info()["id"]
 
     name = os.path.join(tempfile.gettempdir(), "test_multiple_borders.txt")
-    cmd = ["import", "-border", "-window", str(wid), "-crop", "5x1+0+4", "-depth", "8", name]
+    cmd = [
+        shutil.which("import"),
+        "-border",
+        "-window",
+        str(wid),
+        "-crop",
+        "5x1+0+4",
+        "-depth",
+        "8",
+        name,
+    ]
     subprocess.run(cmd, env={"DISPLAY": xmanager.display})
 
-    with open(name, "r") as f:
+    with open(name) as f:
         data = f.readlines()
     os.unlink(name)
 
@@ -752,3 +849,227 @@ def test_net_wm_state_focused(xmanager, conn):
     xmanager.kill_window(two)
     assert_state_focused(wid1, True)
     xmanager.kill_window(one)
+
+
+@manager_config
+def test_window_stacking_order(xmanager):
+    """Test basic window stacking controls."""
+    conn = xcbq.Connection(xmanager.display)
+
+    def _wnd(name):
+        return xmanager.c.window[{w["name"]: w["id"] for w in xmanager.c.windows()}[name]]
+
+    def _clients():
+        root = conn.default_screen.root.wid
+        q = conn.conn.core.QueryTree(root).reply()
+        stack = list(q.children)
+        wins = [(w["name"], stack.index(w["id"])) for w in xmanager.c.windows()]
+        wins.sort(key=lambda x: x[1])
+        return [x[0] for x in wins]
+
+    xmanager.test_window("one")
+    xmanager.test_window("two")
+    xmanager.test_window("three")
+    xmanager.test_window("four")
+    xmanager.test_window("five")
+
+    # We're testing 3 "layers"
+    # BELOW, 'everything else', ABOVE
+
+    # New windows added on top of each other
+    assert _clients() == ["one", "two", "three", "four", "five"]
+
+    # Moving above/below moves above/below next client in the layer
+    _wnd("one").move_up()
+    assert _clients() == ["two", "one", "three", "four", "five"]
+    _wnd("four").move_up()
+    assert _clients() == ["two", "one", "three", "five", "four"]
+    _wnd("one").move_down()
+    assert _clients() == ["one", "two", "three", "five", "four"]
+
+    # Keeping above/below moves client to ABOVE/BELOW layer
+    # When moving to ABOVE, client will be placed at top of that layer
+    # When moving to BELOW, client will be placed at bottom of layer
+
+    # BELOW: None, ABOVE: two
+    _wnd("two").keep_above()
+    assert _clients() == ["one", "three", "five", "four", "two"]
+    _wnd("five").move_up()
+    assert _clients() == ["one", "three", "four", "five", "two"]
+
+    # BELOW: three, ABOVE: two
+    _wnd("three").keep_below()
+    assert _clients() == ["three", "one", "four", "five", "two"]
+    _wnd("four").move_down()
+    assert _clients() == ["three", "four", "one", "five", "two"]
+
+    # BELOW: four, three, ABOVE: two
+    _wnd("four").keep_below()
+    assert _clients() == ["four", "three", "one", "five", "two"]
+
+    # BELOW: four, three, ABOVE: two, one
+    _wnd("one").keep_above()
+    assert _clients() == ["four", "three", "five", "two", "one"]
+    _wnd("five").move_up()
+    assert _clients() == ["four", "three", "five", "two", "one"]
+    _wnd("five").move_down()
+    assert _clients() == ["four", "three", "five", "two", "one"]
+
+    # BELOW: two, four, three, ABOVE: one
+    _wnd("two").keep_below()
+    assert _clients() == ["two", "four", "three", "five", "one"]
+
+    # BELOW: two, three, ABOVE: one, four
+    _wnd("four").keep_above()
+    assert _clients() == ["two", "three", "five", "one", "four"]
+
+    # BELOW: two, three, ABOVE: one
+    _wnd("four").keep_above()
+    assert _clients() == ["two", "three", "five", "four", "one"]
+    _wnd("five").move_up()
+    assert _clients() == ["two", "three", "four", "five", "one"]
+
+    # BELOW: two, three, ABOVE: None
+    _wnd("one").keep_above()
+    assert _clients() == ["two", "three", "four", "five", "one"]
+
+    # BELOW: two, ABOVE: None
+    _wnd("three").keep_below()
+    assert _clients() == ["two", "three", "four", "five", "one"]
+    _wnd("one").move_down()
+    assert _clients() == ["two", "three", "four", "one", "five"]
+
+    # BELOW: None ABOVE: None
+    _wnd("two").keep_below()
+    assert _clients() == ["two", "three", "four", "one", "five"]
+
+    # BELOW: three, ABOVE: None
+    _wnd("three").keep_below()
+    assert _clients() == ["three", "two", "four", "one", "five"]
+    _wnd("two").move_down()
+    assert _clients() == ["three", "two", "four", "one", "five"]
+    _wnd("one").move_down()
+    assert _clients() == ["three", "two", "one", "four", "five"]
+
+    _wnd("two").move_to_top()
+    assert _clients() == ["three", "one", "four", "five", "two"]
+
+    # three is kept_below so moving to bottom is still above that
+    _wnd("five").move_to_bottom()
+    assert _clients() == ["three", "five", "one", "four", "two"]
+
+    # three is the only window kept_below so this will have no effect
+    _wnd("three").move_to_top()
+    assert _clients() == ["three", "five", "one", "four", "two"]
+
+    # Keep three above everything else
+    _wnd("three").keep_above()
+    assert _clients() == ["five", "one", "four", "two", "three"]
+
+    # This should have no effect as it's the only window kept_above
+    _wnd("three").move_to_bottom()
+    assert _clients() == ["five", "one", "four", "two", "three"]
+
+
+@manager_config
+def test_floats_kept_above(xmanager):
+    """Test config option to pin floats to a higher level."""
+    conn = xcbq.Connection(xmanager.display)
+
+    def _wnd(name):
+        return xmanager.c.window[{w["name"]: w["id"] for w in xmanager.c.windows()}[name]]
+
+    def _clients():
+        root = conn.default_screen.root.wid
+        q = conn.conn.core.QueryTree(root).reply()
+        stack = list(q.children)
+        wins = [(w["name"], stack.index(w["id"])) for w in xmanager.c.windows()]
+        wins.sort(key=lambda x: x[1])
+        return [x[0] for x in wins]
+
+    xmanager.test_window("one", floating=True)
+    xmanager.test_window("two")
+
+    # Confirm floating window is above window that was opened later
+    assert _clients() == ["two", "one"]
+
+    # Open a different floating window. This should be above the first floating one.
+    xmanager.test_window("three", floating=True)
+    assert _clients() == ["two", "one", "three"]
+
+
+@manager_config
+def test_fullscreen_on_top(xmanager):
+    """Test fullscreen, focused windows are on top."""
+    conn = xcbq.Connection(xmanager.display)
+
+    def _wnd(name):
+        return xmanager.c.window[{w["name"]: w["id"] for w in xmanager.c.windows()}[name]]
+
+    def _clients():
+        root = conn.default_screen.root.wid
+        q = conn.conn.core.QueryTree(root).reply()
+        stack = list(q.children)
+        wins = [(w["name"], stack.index(w["id"])) for w in xmanager.c.windows()]
+        wins.sort(key=lambda x: x[1])
+        return [x[0] for x in wins]
+
+    xmanager.test_window("one", floating=True)
+    xmanager.test_window("two")
+
+    # window "one" is kept_above, "two" is norm
+    assert _clients() == ["two", "one"]
+
+    # A fullscreen, focused window should display above windows that are "kept above"
+    _wnd("two").enable_fullscreen()
+    _wnd("two").focus()
+    assert _clients() == ["one", "two"]
+
+    # Focusing the other window should cause the fullscreen window to drop from the highest layer
+    _wnd("one").focus()
+    assert _clients() == ["two", "one"]
+
+    # Disabling fullscreen will put the window below the "kept above" window, even if it has focus
+    _wnd("two").focus()
+    _wnd("two").toggle_fullscreen()
+    assert _clients() == ["two", "one"]
+
+
+class UnpinFloatsConfig(ManagerConfig):
+    # New floating windows not set to "keep_above"
+    floats_kept_above = False
+
+
+# Floating windows should be moved above tiled windows when first floated, regardless
+# of whether `floats_kept_above` is True
+@pytest.mark.parametrize("xmanager", [ManagerConfig, UnpinFloatsConfig], indirect=True)
+def test_move_float_above_tiled(xmanager):
+    conn = xcbq.Connection(xmanager.display)
+
+    def _wnd(name):
+        return xmanager.c.window[{w["name"]: w["id"] for w in xmanager.c.windows()}[name]]
+
+    def _clients():
+        root = conn.default_screen.root.wid
+        q = conn.conn.core.QueryTree(root).reply()
+        stack = list(q.children)
+        wins = [(w["name"], stack.index(w["id"])) for w in xmanager.c.windows()]
+        wins.sort(key=lambda x: x[1])
+        return [x[0] for x in wins]
+
+    xmanager.test_window("one")
+    xmanager.test_window("two")
+    xmanager.test_window("three")
+    assert _clients() == ["one", "two", "three"]
+
+    _wnd("two").toggle_floating()
+    assert _clients() == ["one", "three", "two"]
+
+
+def test_multiple_wm_types(xmanager):
+    conn = xcbq.Connection(xmanager.display)
+    w = conn.create_window(50, 50, 50, 50)
+    normal = conn.atoms["_NET_WM_WINDOW_TYPE_NORMAL"]
+    kde_override = conn.atoms["_KDE_NET_WM_WINDOW_TYPE_OVERRIDE"]
+    w.set_property("_NET_WM_WINDOW_TYPE", [kde_override, normal])
+    assert w.get_wm_type() == "normal"
