@@ -13,14 +13,15 @@ The window will close itself if it receives any key or button press events.
 """
 # flake8: noqa
 
-# This is needed otherwise the window will use any Wayland session it can find even if
-# WAYLAND_DISPLAY is not set.
 import os
 
-if os.environ.get("WAYLAND_DISPLAY"):
-    os.environ["GDK_BACKEND"] = "wayland"
-else:
-    os.environ["GDK_BACKEND"] = "x11"
+if not os.environ.get("GDK_BACKEND"):
+    # This is needed otherwise the window will use any Wayland session
+    # it can find even if WAYLAND_DISPLAY is not set.
+    if os.environ.get("WAYLAND_DISPLAY"):
+        os.environ["GDK_BACKEND"] = "wayland"
+    else:
+        os.environ["GDK_BACKEND"] = "x11"
 
 # Disable GTK ATK bridge, which appears to trigger errors with e.g. test_strut_handling
 # https://wiki.gnome.org/Accessibility/Documentation/GNOME2/Mechanics
@@ -167,6 +168,7 @@ if __name__ == "__main__":
     if "urgent_hint" in sys.argv:
 
         def gtk_set_urgency_hint(*args):
+            # Works only when GDK_BACKEND is x11
             win.set_urgency_hint(True)
 
         # Time before changing urgency
