@@ -1,31 +1,3 @@
-# Copyright (c) 2011 Florian Mounier
-# Copyright (c) 2011 Anshuman Bhaduri
-# Copyright (c) 2012-2014 Tycho Andersen
-# Copyright (c) 2013 xarvh
-# Copyright (c) 2013 Craig Barnes
-# Copyright (c) 2014 Sean Vig
-# Copyright (c) 2014 Adi Sieker
-# Copyright (c) 2014 Sebastien Blot
-# Copyright (c) 2020 Mikel Ward
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
-
 import logging
 from pathlib import Path
 
@@ -428,7 +400,10 @@ def test_spawn_list(manager):
 
 
 @manager_config
-def test_spawn_in_group(manager):
+def test_spawn_in_group(manager, backend_name):
+    if backend_name == "wayland":
+        pytest.skip("TODO: X11 only for now.")
+
     @Retry(ignore_exceptions=(AssertionError,))
     def wait_for_window(empty=False):
         assert (len(manager.c.windows()) > 0) is not empty
@@ -1110,24 +1085,36 @@ def test_labelgroup(manager):
 
 
 @manager_config
-def test_change_loglevel(manager):
+def test_change_loglevel(manager, backend_name):
     assert manager.c.loglevel() == logging.INFO
     assert manager.c.loglevelname() == "INFO"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_INFO == lib.wlr_log_get_verbosity()")[1] == "True"
     manager.c.debug()
     assert manager.c.loglevel() == logging.DEBUG
     assert manager.c.loglevelname() == "DEBUG"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_DEBUG == lib.wlr_log_get_verbosity()")[1] == "True"
     manager.c.info()
     assert manager.c.loglevel() == logging.INFO
     assert manager.c.loglevelname() == "INFO"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_INFO == lib.wlr_log_get_verbosity()")[1] == "True"
     manager.c.warning()
     assert manager.c.loglevel() == logging.WARNING
     assert manager.c.loglevelname() == "WARNING"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_ERROR == lib.wlr_log_get_verbosity()")[1] == "True"
     manager.c.error()
     assert manager.c.loglevel() == logging.ERROR
     assert manager.c.loglevelname() == "ERROR"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_ERROR == lib.wlr_log_get_verbosity()")[1] == "True"
     manager.c.critical()
     assert manager.c.loglevel() == logging.CRITICAL
     assert manager.c.loglevelname() == "CRITICAL"
+    if backend_name == "wayland":
+        assert manager.c.core.eval("lib.WLR_SILENT == lib.wlr_log_get_verbosity()")[1] == "True"
 
 
 def test_switch_groups_cursor_warp(manager_nospawn):
