@@ -1,29 +1,10 @@
-# Copyright (c) 2021-22 elParaguayo
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
 import pytest
 
 import libqtile.bar
 import libqtile.config
 from libqtile.command.base import expose_command
 from libqtile.widget import Spacer, TextBox
-from libqtile.widget.base import ThreadPoolText, _Widget
+from libqtile.widget.base import BackgroundPoll, _Widget
 from test.helpers import BareConfig, Retry
 
 
@@ -50,7 +31,7 @@ class TimerWidget(_Widget):
         return len(active)
 
 
-class PollingWidget(ThreadPoolText):
+class PollingWidget(BackgroundPoll):
     poll_count = 0
 
     def poll(self):
