@@ -1,22 +1,3 @@
-# Copyright (c) 2024 elParaguayo
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-# SOFTWARE.
 import datetime
 import sys
 from importlib import reload
@@ -108,8 +89,10 @@ def test_vclock_extra_lines(vclock_manager):
     [
         dict(fontsize=[10]),  # too few
         dict(fontsize=[10, 20, 30, 40]),  # too many
+        dict(fontsize=[10, "fff"]),  # mix values
         dict(foreground=["fff"]),  # too few
         dict(foreground=["fff"] * 4),  # too many
+        dict(foreground=["fff", 10]),  # mix values
     ],
     indirect=True,
 )
